package com.energy.online.data.service.listener;

import com.alibaba.fastjson.JSON;
import com.beust.jcommander.internal.Lists;
import com.energy.online.data.service.po.MeasurementTempPO;
import com.energy.online.data.service.service.J104ClinetService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.openmuc.j60870.ASdu;
import org.openmuc.j60870.ConnectionEventListener;
import org.openmuc.j60870.ie.InformationElement;
import org.openmuc.j60870.ie.InformationObject;
import org.springframework.http.*;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.time.Instant;
import java.util.*;

@Slf4j
public class J60870ClientListener implements ConnectionEventListener {

    private J104ClinetService j104ClinetService;

    private String addrerss;

    private Integer confId;

    public J60870ClientListener(J104ClinetService j104ClinetService, String addrerss, Integer confId) {
        this.j104ClinetService = j104ClinetService;
        this.addrerss = addrerss;
        this.confId = confId;
    }

    public final String title = "104协议数据服务 - ";


    /**
     * 监听从站发来的数据
     *
     * @param aSdu
     */
    @Override
    public void newASdu(ASdu aSdu) {
//        log.info(this.title + "J60870ClientListener NewASdu Get ：" + aSdu.toString());
        List<InformationObject> list = new ArrayList<>(Arrays.asList(aSdu.getInformationObjects()));
        //  获取当前时刻,验证是否是同一秒数据，数据进行合并
        Instant now = Instant.now();
        // 获取当前时刻的秒数
        long epochSecond = now.getEpochSecond();
        Long receiptTime = j104ClinetService.getMeasurementTime(confId);
        long receiptTimeLong = receiptTime == null ? 0 : receiptTime.longValue();
        log.info(this.title + "J60870ClientListener NewASdu Get time ====> " + epochSecond+"，接收数据大小："+list.size());
        // 数据库最新数据秒时间等于0或者与当前秒时间相等
        if (receiptTimeLong == 0 || epochSecond == receiptTimeLong) {
            // 保存临时表
            saveMeasurementTemp(list, confId, epochSecond);
        }
        // 当前秒时间大于数据库最新秒时间并且数据库不等于0
        if (epochSecond > receiptTimeLong && receiptTimeLong != 0) {
            int commonAddress = aSdu.getCommonAddress();
            List<MeasurementTempPO> measurementTempPOList = j104ClinetService.getMeasurementTemp(receiptTimeLong, confId);
            log.info(JSON.toJSONString("时间秒数" + receiptTimeLong +  ", 数据长度：" + measurementTempPOList.size()));

            Map<String, String> dataMap = new HashMap<>();
            //动态创建表并入库
            for (MeasurementTempPO po : measurementTempPOList) {
                Integer informationObjectAddress = po.getDataPoin();
                String informationObjectToString = po.getReceiptData();
                dataMap.put(String.valueOf(informationObjectAddress), informationObjectToString);
            }
            j104ClinetService.processingDat(receiptTimeLong, dataMap, confId, commonAddress, addrerss);
            // 删除上一秒数据
            j104ClinetService.initDelMeasurementTemp();
            // 保存临时表
            saveMeasurementTemp(list, confId, epochSecond);
        }
    }


    private void saveMeasurementTemp(List<InformationObject> informationObjects, Integer confId, long epochSecond) {
        MeasurementTempPO measurementTempPO = null;
        List<MeasurementTempPO> list = Lists.newArrayList();
        for (InformationObject item : informationObjects) {
            // 秒数据入库
            measurementTempPO = new MeasurementTempPO();
            measurementTempPO.setConfId(confId);
            measurementTempPO.setReceiptDate(new Date());
            measurementTempPO.setReceiptTime(epochSecond);
            measurementTempPO.setDataPoin(item.getInformationObjectAddress());
            measurementTempPO.setReceiptData(toString(item));
            list.add(measurementTempPO);
        }
        j104ClinetService.saveMeasurementTemp(list);
    }


    /**
     * 监听连接关闭
     *
     * @param cause
     */
    @Override
    public void connectionClosed(IOException cause) {
        log.debug(this.title + "J60870ClientListener ConnectionClosed ：" + cause.getMessage());
    }

    /**
     * ASdu类方法重写
     *
     * @param aSdu
     * @return
     */
    public String toString(ASdu aSdu) {
        List<String> strList = new ArrayList<>();
        if (aSdu.getInformationObjects() != null) {
            InformationObject[] informationObjects = aSdu.getInformationObjects();
            String str;
            for (int i = 0; i < informationObjects.length; ++i) {
                str = toString(informationObjects[i]);
                if (str != null) {
                    strList.add(str);
                }
            }
        }
        return strList.size() > 0 ? StringUtils.join(strList, "***") : null;
    }

    /**
     * InformationObject类方法重写
     *
     * @param informationObjectRecord
     * @return
     */
    public String toString(InformationObject informationObjectRecord) {
        String str;
        InformationElement[] informationElementSet;
        List<String> strList = new ArrayList<>();
        String split = "Short float value:";
        InformationElement[][] valData = informationObjectRecord.getInformationElements();
        if (valData.length > 0) {
            for (int i = 0; i < valData.length; ++i) {
                informationElementSet = valData[i];
                for (int ii = 0; ii < informationElementSet.length; ++ii) {
                    /**
                     * 目测可能出现的情况：
                     * Short float value: 18.2
                     * Quality, overflow: false, blocked: false, substituted: false, not topical: false, invalid: false
                     */
                    str = informationElementSet[ii].toString();
                    if (str.contains(split)) {
                        strList.add(str.split(split)[1].trim());
                    }
                }
            }
        }
        return strList.size() > 0 ? StringUtils.join(strList, ",") : null;
    }

    /**
     * 发送POST请求
     *
     * @param url
     * @param params
     * @return
     */
    public String sendPOSTRequest(String url, MultiValueMap<String, String> params) {
        RestTemplate client = new RestTemplate();
        HttpHeaders headers = new HttpHeaders();
        HttpMethod method = HttpMethod.POST;
        // 以表单的方式提交
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        // 将请求头部和参数合成一个请求
        HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<>(params, headers);
        // 执行HTTP请求，将返回的结构使用String类格式化
        ResponseEntity<String> response = client.exchange(url, method, requestEntity, String.class);
        return response.getBody();
    }


}

