package com.energy.online.data.service.listener;

import com.energy.online.data.service.service.J104ClinetService;
import lombok.extern.slf4j.Slf4j;
import org.openmuc.j60870.ClientConnectionBuilder;
import org.openmuc.j60870.Connection;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;

@Slf4j
public class JClient implements Callable<String> {

    public static String title = "104协议数据服务 - ";

    private String addrerss;

    private Integer confId;

    int port = 2404;

    int time = 3000;
    //监听类，后面代码会给出
    private J60870ClientListener listener;
    //连接类
    private Connection connection;

    //连接参数，可以配置的参数有好多，比如t0,t1,t2等等
    private ClientConnectionBuilder clientConnectionBuilder;

    private J104ClinetService j104ClinetService;

    @Override
    public String call() throws Exception {

        //主站与从站连接
        connection = clientConnectionBuilder.build();
        //配置数据回调类
        listener = new J60870ClientListener(this.j104ClinetService, this.addrerss, this.confId);

        connection.startDataTransfer(listener);

        log.info(title + "J60870Client Run Success By Listener");

        return "Success";
    }


    /**
     * @param addrerss
     */
    public JClient(String addrerss, Integer confId, J104ClinetService j104ClinetService) throws UnknownHostException {
        this.addrerss = addrerss;
        this.confId = confId;
        this.j104ClinetService = j104ClinetService;
        init();
    }

    /**
     * @param addrerss
     * @param port
     */
    public JClient(String addrerss, Integer confId, int port, J104ClinetService j104ClinetService) throws UnknownHostException {
        this.addrerss = addrerss;
        this.port = port;
        this.confId = confId;
        this.j104ClinetService = j104ClinetService;
        init();
    }

    /**
     * @param addrerss
     * @param port
     * @param time
     */
    public JClient(String addrerss, Integer confId, int port, int time, J104ClinetService j104ClinetService) throws UnknownHostException {
        this.addrerss = addrerss;
        this.port = port;
        this.confId = confId;
        this.time = time;
        this.j104ClinetService = j104ClinetService;
        init();
    }

    /**
     * 初始化
     */
    public void init() throws UnknownHostException {

        //获取server端、从站的ip地址对象，这里使用本地ip
        InetAddress address = InetAddress.getByName(addrerss);
        //创建连接参数对象
        clientConnectionBuilder = new ClientConnectionBuilder(address);

        //设置socket的连接超时时间
        clientConnectionBuilder.setConnectionTimeout(time);

        //设置socket的连接超时时间
        clientConnectionBuilder.setPort(port);

        log.info(title + "J60870Client Init Success By Host " + addrerss);
    }

    /**
     * 关闭连接
     */
    public void disconnect() {
        try {

            this.connection.close();

            log.info(title + "J60870Client Disconnect Success By Close");

        } catch (Exception e) {

            e.printStackTrace();

            log.info(title + "J60870Client Disconnect Exception By Close");
        }
    }
}
