package com.energy.online.data.service.listener;

import cn.hutool.core.util.StrUtil;
import com.energy.online.data.service.po.MeasurementConfPO;
import com.energy.online.data.service.service.J104ClinetService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import static java.util.concurrent.Executors.*;

@Slf4j
@Component
public class ListenerMainService implements ApplicationListener<ContextRefreshedEvent> {

    @Autowired
    private J104ClinetService j104ClinetService;


    @Override
    public void onApplicationEvent(ContextRefreshedEvent contextRefreshedEvent) {
        log.info("启动监听到 J60870Client...");
        // 清空临时表数据
//        j104ClinetService.initDelMeasurementTemp();
        // 查询配置表
        List<MeasurementConfPO> measurementConfPOS = j104ClinetService.getMeasurementConfList();
        int taskNum = measurementConfPOS.size();
        // 创建一个固定大小为size的线程池
        ExecutorService executor = createOptimizedThreadPool(measurementConfPOS.size());
        //=========================多线程,提交任务到线程池================================
        for (int i = 0; i < taskNum; i++) {
            final int taskNumber = i;
            MeasurementConfPO measurementConfPO = measurementConfPOS.get(i);
            System.out.println("Task " + taskNumber + " is running in thread: " + Thread.currentThread().getName());
            Future<String> future = null;
            try {
                future = executor.submit(new JClient(measurementConfPO.getMeasurementIp(),measurementConfPO.getId(),j104ClinetService));
            } catch (UnknownHostException e) {
                e.printStackTrace();
            }finally {
                executor.shutdown();
            }
            try {
                String result = future.get();
                System.out.println("Task completed successfully with result: " + result);
            } catch (InterruptedException e) {
                System.out.println("Current thread was interrupted while waiting for the task to complete.");
                Thread.currentThread().interrupt(); // 恢复中断状态
            } catch (ExecutionException e) {
                System.out.println("An exception was thrown by the task.");
                Throwable cause = e.getCause();
                if (cause instanceof RuntimeException) {
                    System.out.println("RuntimeException: " + cause.getMessage());
                } else if (cause instanceof IOException) {
                    System.out.println("IOException: " + cause.getMessage());
                } else {
                    System.out.println("Other exception: " + cause.getClass().getName() + ", message: " + cause.getMessage());
                }
            }finally {
                executor.shutdown();
            }
            log.info("监听地址：" + measurementConfPO.getMeasurementIp());
        }

        //=========================多进程================================
        //=========================单进程================================
//        String[] apponitPath = apponitPathList.get(0).split(":");
//        executor.submit(new JClient(apponitPath[0],apponitPath[1],j104ClinetService));
//        log.info("监听地址：" + "127.0.0.1" );

    }

    /**
     * 返回线程数
     * @param taskCount
     * @return
     */
    private ExecutorService createOptimizedThreadPool(int taskCount) {
        // 获取可用处理器核心数
        int availableProcessors = Runtime.getRuntime().availableProcessors();

        // 根据任务类型和系统资源选择合适的线程数
        // 这里假设任务是IO密集型的，因此我们将线程数设置为核心数的两倍
        int threadPoolSize = Math.max(availableProcessors * 2, taskCount);

        return newFixedThreadPool(threadPoolSize);
    }

}
