package com.energy.online.data.service.service.impl;


import com.alibaba.fastjson.JSON;
import com.energy.online.data.service.dto.MeasurementDto;
import com.energy.online.data.service.mapper.MeasurementConfMapper;
import com.energy.online.data.service.mapper.MeasurementMapper;
import com.energy.online.data.service.mapper.MeasurementTempMapper;
import com.energy.online.data.service.po.MeasurementConfPO;
import com.energy.online.data.service.po.MeasurementTempPO;
import com.energy.online.data.service.service.J104ClinetService;
import com.energy.online.data.service.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import tk.mybatis.mapper.entity.Example;

import java.beans.Transient;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class J104ClinetServiceImpl implements J104ClinetService {

    @Autowired
    private MeasurementMapper measurementMapper;

    @Autowired
    private MeasurementTempMapper measurementTempMapper;

    @Autowired
    private MeasurementConfMapper measurementConfMapper;



    @Override
    @Transactional(rollbackFor = Exception.class)
    public void processingDat(long receiptTimeLong,Map<String, String> dataMap, Integer confId, Integer commonAddress, String addrerss) {
        // 当年年月日时间表名
        String dateTime = DateUtils.convertDate2StringYMD(new Date());
        String tableName = dateTime + "_" +  confId;
        // 判断表是否存在
        if (measurementMapper.selectTableCountByTableName(tableName) <= 0) {
            measurementMapper.createTable(tableName);
            log.info("动态创建表 ====> " + tableName);
        }
        if (CollectionUtils.isEmpty(dataMap)) {
            log.error(" ==== dataMap数据为空！ ==== ");
            return;
        }
        MeasurementDto dto = new MeasurementDto();
        dto.setReceiveTime(new Date());
        dto.setCommonAddress(commonAddress);
        dto.setTime(receiptTimeLong);
        dto.setInformationObjectData(JSON.toJSONString(dataMap));
        dto.setInformationObjectSize(dataMap.size());
        dto.setInformationIp(addrerss);
        if (measurementMapper.insertTableData(dto, tableName) > 0) {
            log.info("104规约导入数据成功 ===>" + dto.getTime() + "秒，时间转换：" + DateUtils.convertDate2StringYMDHMS(dto.getReceiveTime()));
        } else {
            log.error("104规约导入数据失败，时间转换：" + DateUtils.convertDate2StringYMDHMS(dto.getReceiveTime()));
        }
    }

    @Override
    @Transactional
    public boolean saveMeasurementTemp(List<MeasurementTempPO> measurementTempPOList) {
        return measurementTempMapper.insertList(measurementTempPOList) > 0;
    }


    @Override
    @Transactional
    public boolean delMeasurementTemp(Long receiptTime,Integer confId) {
        Example queryExample = new Example(MeasurementTempPO.class);
        Example.Criteria criteria = queryExample.createCriteria();
        criteria.andEqualTo("confId",confId);
        criteria.andEqualTo("receiptTime",receiptTime);
        return measurementTempMapper.deleteByExample(queryExample) > 0;
    }


    @Override
    public List<MeasurementTempPO> getMeasurementTemp(Long receiptTime,Integer confId) {
        MeasurementTempPO po = new MeasurementTempPO();
        po.setReceiptTime(receiptTime);
        po.setConfId(confId);
        return measurementTempMapper.select(po);
    }

    @Override
    public Long getMeasurementTime(Integer confId) {
        return measurementMapper.selectMeasurementTempByid(confId);
    }

    @Override
    public List<MeasurementConfPO> getMeasurementConfList() {
        return measurementConfMapper.selectAll();
    }


    @Override
    @Transactional
    public void initDelMeasurementTemp() {
        measurementMapper.truncateMeasurementTempTable();
    }


}
